@@ -28,6 +28,8 @@ DESC
     [ UNCOMMON_VALID, '00size' => "chunks size prefixed with 00" ],
     [ UNCOMMON_VALID, 'ucsize' => "upper case size" ],
     [ UNCOMMON_VALID, '0ucsize' => "upper case size prefix with 0" ],
+    [ INVALID, '32-size' => "negative size for 32bit uint" ],
+    [ INVALID, '64-size' => "negative size for 64bit uint" ],
     [ INVALID, 'size-space' => "size followed by space" ],
     [ INVALID, 'size-tab' => "size followed by tab" ],
     [ INVALID, 'size-cr' => "size followed by <CR>" ],
@@ -52,9 +54,12 @@ DESC
     [ INVALID, '\013-size' => "size prefixed by char \\v" ],
     [ INVALID, '\014-size' => "size prefixed by char \\f" ],
     [ INVALID, 'xspace-size' => "size prefixed by char 'x' and space" ],
+    [ INVALID, '\053-size' => "size prefixed by char '+'" ],
+    [ INVALID, '\060\170-size' => "size prefixed by '0x'" ],
     [ UNCOMMON_VALID, 'final=00' => 'final chunk size "00"' ],
     [ UNCOMMON_VALID, 'final=00000000000000000000' => 'final chunk size "00000000000000000000"' ],
     [ INVALID, 'final=0x' => 'final chunk size "0x"' ],
+    [ INVALID, 'final=Foo' => 'final chunk size "Foo"' ],
     [ INVALID, 'finalchunk=0\012' => 'final chunk just "0\n"' ],
     [ INVALID, 'finalchunk=0\015' => 'final chunk just "0\r"' ],
     [ INVALID, 'finalchunk=0' => 'final chunk just "0"' ],
@@ -112,6 +117,17 @@ DESC
     [ INVALID, 'chunked,http/1.0' => 'Chunked Header and http/1.0. Served chunked.'],
     [ INVALID, 'chunked,clen,http/1.0,do_clen' => 'Chunked Header and http/1.0. Not served chunked.'],
 
+    [ INVALID, 'chunked,HTTP/0.1,gzip' => 'Chunked Header and HTTP/0.1. Served chunked and with gzip.'],
+    [ INVALID, 'chunked,clen,HTTP/0.1,gzip,do_clen' => 'Chunked Header and HTTP/0.1. Not served chunked but with gzip.'],
+    [ INVALID, 'chunked,HTTP/01.1,gzip' => 'Chunked Header and HTTP/01.1. Served chunked and with gzip.'],
+    [ INVALID, 'chunked,clen,HTTP/01.1,gzip,do_clen' => 'Chunked Header and HTTP/01.1. Not served chunked but with gzip.'],
+    [ INVALID, 'chunked,HTTP/11.01,gzip' => 'Chunked Header and HTTP/11.01. Served chunked and with gzip.'],
+    [ INVALID, 'chunked,clen,HTTP/11.01,gzip,do_clen' => 'Chunked Header and HTTP/11.01. Not served chunked but with gzip.'],
+    [ INVALID, 'chunked,HTTP/11.10,gzip' => 'Chunked Header and HTTP/11.10. Served chunked and with gzip.'],
+    [ INVALID, 'chunked,clen,HTTP/11.10,gzip,do_clen' => 'Chunked Header and HTTP/11.10. Not served chunked but with gzip.'],
+    [ INVALID, 'chunked,HTTP/9.9,gzip' => 'Chunked Header and HTTP/9.9. Served chunked and with gzip.'],
+    [ INVALID, 'chunked,clen,HTTP/9.9,gzip,do_clen' => 'Chunked Header and HTTP/9.9. Not served chunked but with gzip.'],
+
     [ 'VALID: valid variations on "chunked" value' ],
     [ VALID, 'chUnked' => 'mixed case "chUnked", served chunked'],
     [ UNCOMMON_VALID,'nl-chunked' => "chunked header with continuation line, served chunked"],
@@ -254,6 +270,10 @@ sub make_response {
 	    $eol =~s{cr}{\r}g;
 	    $eol =~s{lf}{\n}g;
 	    $chunkmod{lineend} = $eol;
+	} elsif (m{^(32|64)-size\z}) {
+	    my $o = ($1 == 64) ? 'ffffffff':'';
+	    $sizefmt = sub { sprintf("-$o%08x", 1+(0xffffffff & ~shift())) };
+	    $hdr .= "Transfer-Encoding: chunked\r\nConnection: close\r\n";
 	} elsif ( m{^(-|space|cr|lf|tab|x|\\[0-7]{3})*(0*)(uc)?size(-|space|cr|lf|tab|x|\\[0-7]{3})*$}) {
 	    $hdr .= "Transfer-Encoding: chunked\r\nConnection: close\r\n";
 	    @chunks = ( $data =~m{(.{1,15})}smg,'') if ! @chunks;
@@ -313,10 +333,10 @@ sub make_response {
 	    $end = sprintf("%x%s%s%s",$last->[0]+10,$last->[2]||'',$nl,$last->[1]);
 	}
 
-	$finalchunk //= "$final$nl$nl";
+	$finalchunk = "$final$nl$nl" if ! defined $finalchunk;
 	$data = join("",map { 
-	    $_->[0] ? sprintf("$sizefmt%s%s%s%s",
-		$_->[0],          # size
+	    $_->[0] ? sprintf("%s%s%s%s%s",
+		ref($sizefmt) ? $sizefmt->($_->[0]): sprintf($sizefmt,$_->[0]), # size
 		$_->[2] || '',    # ext
 		$nl,
 		$_->[1],
