@@ -44,6 +44,15 @@ DESC
     [ INVALID, 'ce:gzip;gzip2p,finish' => 'content-encoding gzip, served gzipped with 2 compressed blocks with finish in between'],
     [ INVALID, 'ce:deflate;deflate2p,finish' => 'content-encoding deflate, served with deflate with 2 compressed blocks with finish in between'],
 
+    [ 'INVALID: only part of data compressed, followed by uncompressed data' ],
+    [ INVALID, 'ce:gzip;gzip2s' => 'content-encoding gzip, first segment compressed with gzip, next uncompressed' ],
+    [ INVALID, 'ce:deflate;deflate2s' => 'content-encoding deflate, first segment compressed with deflate, next uncompressed' ],
+    [ INVALID, 'ce:deflate;zlib2s' => 'content-encoding deflate, first segment compressed with zlib, next uncompressed' ],
+    [ INVALID, 'ce:deflate;pkt:zlib+deflate' => 'content-encoding deflate, first segment compressed with zlib but ADLER32 removed, next with deflate in new TCP packet' ],
+    [ INVALID, 'ce:deflate;chk:zlib+deflate' => 'content-encoding deflate, first segment compressed with zlib but ADLER32 removed, next with deflate in new chunk with chunked encoding' ],
+    [ INVALID, 'ce:deflate;pkt:zlib+deflate+deflate' => 'content-encoding deflate, first segment compressed with zlib but ADLER32 removed, next two with deflate in new TCP packets' ],
+    [ INVALID, 'ce:deflate;chk:zlib+deflate+deflate' => 'content-encoding deflate, first segment compressed with zlib but ADLER32 removed, next two with deflate in new chunk with chunked encoding' ],
+
     [ 'VALID: lzma (supported by at least Opera)' ],
     [ UNCOMMON_VALID, 'ce:lzma;lzma1' => 'content-encoding lzma, lzma1 (lzma_alone) encoded'],
 
@@ -199,6 +208,8 @@ DESC
     [ INVALID,'ce:gzip;gzip;replace:3,1|02;replace:10,0=0000', 'set flag FHCRC and add CRC with 0'],
     [ UNCOMMON_VALID,'ce:gzip;gzip;replace:3,1|04;replace:10,0=0000', 'set flag FEXTRA and extra part with XLEN 0'],
     [ UNCOMMON_VALID,'ce:gzip;gzip;replace:3,1|04;replace:10,0=05004170010000', 'set flag FEXTRA and extra part with XLEN 5'],
+    [ INVALID,'ce:gzip;gzip;replace:3,1|04;replace:10,0=0500', 'set flag FEXTRA and XLEN 5 but no extra part'],
+    [ INVALID,'ce:gzip;gzip-payload-as-extra', 'gzip, but hide the real (deflate) payload inside the EXTRA part'],
     [ UNCOMMON_VALID,'ce:gzip;gzip;replace:3,1|08;replace:10,0=2000', 'set flag FNAME and add short file name'],
     [ UNCOMMON_VALID,'ce:gzip;gzip;replace:3,1|10;replace:10,0=2000', 'set flag FCOMMENT and add short comment'],
     [ INVALID,'ce:gzip;gzip;replace:3,1|20', 'set flag reserved bit 5'],
@@ -253,6 +264,8 @@ sub make_response {
     my $version = '1.1';
     my $clen_extend;
     my $body_prefix = '';
+    my $te = 'clen';
+    my @data; # preferred against $data if given
     for (split(';',$spec)) {
 	if ($_ eq 'ce:rfc2047-deflate') {
 	    $hdr .= "Content-Encoding: =?UTF-8?B?ZGVmbGF0ZQo=?=\r\n";
@@ -267,7 +280,31 @@ sub make_response {
 	    $hdr .= "Connection: close\r\n" if $changed;
 	    $hdr .= $field eq 'ce' ? 'Content-Encoding:':'Transfer-Encoding:';
 	    $hdr .= "$v\r\n";
-	} elsif ( m{^(?:(gzip)|deflate|(zlib))(?:(\d+)p)?(?:,(sync|partial|block|full|finish))?$} ) {
+	} elsif ( m{^(pkt|chk):zlib\+deflate(\+deflate)?\z} ) {
+	    # [zlib-header][deflate][more-deflate]....
+	    # zlib will return with Z_DATA_ERROR when trying to process
+	    # more-deflate because it actually expected the correct ADLER32
+	    # checksum there. Browsers will then assume that this should be
+	    # raw-deflate instead and retry with an edded zlib header.
+	    # With some browsers this process can be repeated.
+
+	    my ($enc,$nchunks) = ($1, $2? 3:2);
+	    my $size = int(length($data)/$nchunks);
+	    @data = ();
+	    for(my $i=0;$i<$nchunks;$i++) {
+		push @data, substr($data,0,$size,'');
+	    }
+	    $data[-1] .= $data; # in case something left
+	    $_ = zlib_compress($_,'deflate') for(@data);
+	    $data[0] = "\x78\x9c".$data[0];
+	    if ($enc eq 'chk') {
+		$te = 'chunked';
+		$data = join("", map {
+		    sprintf("%x\r\n%s\r\n",length($_),$_)
+		} (@data,''));
+		@data = ();
+	    }
+	} elsif ( m{^(?:(gzip)|deflate|(zlib))(?:(\d+)([ps]))?(?:,(sync|partial|block|full|finish))?$} ) {
 	    my $zlib = Compress::Raw::Zlib::Deflate->new(
 		-WindowBits => $1 ? WANT_GZIP : $2 ? +MAX_WBITS() : -MAX_WBITS(),
 		-AppendOutput => 1,
@@ -277,14 +314,17 @@ sub make_response {
 	    while ($data ne '') {
 		push @chunks,substr($data,0,$size,'')
 	    }
+	    my $plain_chunk = '';
+	    $plain_chunk = join('',splice(@chunks,1)) if $4 && $4 eq 's';
+
 	    my $flush =
-		! $4 ? Z_FULL_FLUSH :
-		$4 eq 'partial' ? Z_PARTIAL_FLUSH :
-		$4 eq 'sync'    ? Z_SYNC_FLUSH :
-		$4 eq 'full'    ? Z_FULL_FLUSH :
-		$4 eq 'block'   ? Z_BLOCK :
-		$4 eq 'finish'  ? Z_FINISH :
-		die $4;
+		! $5 ? Z_FULL_FLUSH :
+		$5 eq 'partial' ? Z_PARTIAL_FLUSH :
+		$5 eq 'sync'    ? Z_SYNC_FLUSH :
+		$5 eq 'full'    ? Z_FULL_FLUSH :
+		$5 eq 'block'   ? Z_BLOCK :
+		$5 eq 'finish'  ? Z_FINISH :
+		die $5;
 	    my $newdata = '';
 	    while (@chunks) {
 		$zlib->deflate( shift(@chunks), $newdata);
@@ -294,7 +334,7 @@ sub make_response {
 		}
 	    }
 	    $zlib->flush($newdata,Z_FINISH);
-	    $data = $newdata;
+	    $data = $newdata . $plain_chunk;
 	} elsif (m{^(lzma[12]|xz)$}) {
 	    my ($lzma,$status) = 
 		$_ eq 'xz'    ? Compress::Raw::Lzma::EasyEncoder->new(AppendOutput => 1)  :
@@ -344,6 +384,37 @@ sub make_response {
 	    } else {
 		die "bad op=$op in '$_'"
 	    }
+
+	} elsif ( $_ eq 'gzip-payload-as-extra') {
+
+	    # header with FEXTRA set
+	    my $gzip = pack("CCCCVCC",0x1f,0x8b,0x8,0b100,0,2,0);
+
+	    # add XLEN + payload with deflate data
+	    my $zlib = Compress::Raw::Zlib::Deflate->new(
+		-WindowBits => -MAX_WBITS(),
+		-AppendOutput => 1,
+	    );
+	    my $newdata = '';
+	    $zlib->deflate($data,$newdata);
+	    $zlib->flush($newdata,Z_FINISH);
+	    $gzip .= pack("v/a*",$newdata);
+
+	    # then add some innocent compressed data
+	    $data = 'This is not what you are looking for.';
+	    $zlib = Compress::Raw::Zlib::Deflate->new(
+		-WindowBits => -MAX_WBITS(),
+		-AppendOutput => 1,
+	    );
+	    $newdata = '';
+	    $zlib->deflate($data,$newdata);
+	    $zlib->flush($newdata,Z_FINISH);
+	    $gzip .= $newdata;
+
+	    # and add the trailer with CRC and length based on the innocent data
+	    $gzip .= pack("VV",Compress::Raw::Zlib::crc32($data),length($data));
+	    $data = $gzip;
+
 	} elsif (m{^clen\+(\d+)$}) {
 	    $clen_extend = $1
 	} elsif ($_ eq 'noclen') {
@@ -357,15 +428,25 @@ sub make_response {
     }
 
     $data = $body_prefix . $data;
+    if (! @data) {
+	@data = $data;
+    } else {
+	$data = join('',@data);
+    }
     my $len = length($data);
-    if (!$clen_extend) {
+    if ($te eq 'chunked') {
+	$hdr .= "Transfer-Encoding: chunked\r\n"
+    } elsif (!$clen_extend) {
 	$hdr = "Content-length: ".length($data)."\r\n".$hdr;
     } elsif ($clen_extend<0) {
 	$hdr = "Connection: close\r\n$hdr";
     } else {
 	$hdr = "Connection: close\r\nContent-length: ".(length($data)+$clen_extend)."\r\n".$hdr;
     }
-    return "HTTP/$version 200 ok\r\n$hdr\r\n$data";
+    return (
+	"HTTP/$version 200 ok\r\n$hdr\r\n$data[0]",
+	(@data>1) ? (@data[1..$#data]):(),
+    );
 }
 
 1;
